@@ -8,6 +8,7 @@
 
     Jenkins execution helper script
 '''
+# pylint: disable=repr-flag-used-in-string,wrong-import-order
 
 # Import python libs
 from __future__ import absolute_import, print_function
@@ -16,7 +17,6 @@ import sys
 import json
 import time
 import pipes
-import time
 import random
 import hashlib
 import socket
@@ -181,13 +181,13 @@ def generate_ssh_keypair(options):
 
     exitcode = run_command(
         'ssh-keygen -b 2048 -C "$(whoami)@$(hostname)-$(date \"+%Y-%m-%dT%H:%M:%S%z\")" '
-        '-f {0} -N \'\' -V -10m:+2h'.format(ssh_key_path),
+        '-f {0} -N \'\' -V -10m:+8h'.format(ssh_key_path),
         options
     )
     if exitcode != 0:
         exitcode = run_command(
             'ssh-keygen -t rsa -b 2048 -C "$(whoami)@$(hostname)-$(date \"+%Y-%m-%dT%H:%M:%S%z\")" '
-            '-f {0} -N \'\' -V -10m:+2h'.format(ssh_key_path),
+            '-f {0} -N \'\' -V -10m:+8h'.format(ssh_key_path),
             options
         )
         if exitcode != 0:
@@ -555,8 +555,8 @@ def bootstrap_parallels_minion(options):
         run_command(_prl_cmd('revert_snapshot', options.vm_source, options.vm_snapshot), options)
         # Get source VM state
         source_state = run_command(_prl_cmd('status', options.vm_source),
-                                            options,
-                                            return_output=True)[0]
+                                   options,
+                                   return_output=True)[0]
 
         # Ensure source VM is stopped since running VMs cannot be cloned
         if 'running' in source_state:
@@ -707,10 +707,9 @@ def prepare_ssh_access(options):
         provider_password = cloud_config['providers']['linode']['linode']['password']
 
         # Using the password, we can construct a data structure for a salt-ssh roster
-        roster_data = {options.vm_name:
-                            {'host': get_minion_ip_address(options, sync=False),
-                            'passwd': provider_password}
-                        }
+        roster_data = {options.vm_name: {
+                        'host': get_minion_ip_address(options, sync=False),
+                        'passwd': provider_password}}
         # FIXME Perms!
         if not os.path.exists('/tmp/.jenkins_ssh'):
             os.mkdir('/tmp/.jenkins_ssh')
@@ -889,7 +888,9 @@ def get_minion_ip_address(options, sync=True):
             if options.ssh_private_address:
                 ip_address = find_private_addr(ip_info[options.vm_name])
             else:
-                ip_address = ip_info[options.vm_name][0]
+                ip_address = ip_info[options.vm_name]
+                if isinstance(ip_address, (list, tuple)):
+                    ip_address = ip_address[0]
             if not ip_address:
                 print_bulleted(
                     options,
@@ -923,52 +924,53 @@ def get_minion_python_executable(options):
 
     sync_minion(options)
 
-    cmd = [
-        'salt',
-        '--out=json',
-        '-l', options.log_level
-    ]
-    if options.no_color:
-        cmd.append('--no-color')
-    cmd.append(options.vm_name)
     if options.windows:
-        cmd.extend(['cmd.which', 'python'])
+        if options.test_with_python3:
+            python_executable = 'C:\\PROGRA~1\\Python35\\python.exe'
+        else:
+            python_executable = 'C:\\Python27\\python.exe'
     else:
-        cmd.extend(['grains.get', 'pythonexecutable'])
-    stdout, stderr, exitcode = run_command(cmd,
-                                           options,
-                                           return_output=True,
-                                           stream_stdout=False,
-                                           stream_stderr=False)
-    if exitcode != 0:
-        print_bulleted(
-            options,
-            'Failed to get the minion python executable. Exit code: {0}'
-            ''.format(exitcode),
-            'RED')
-        sys.exit(exitcode)
-
-    if not stdout.strip():
-        print_bulleted(
-            options,
-            'Failed to get the minion python executable (no output)',
-            'RED')
-        sys.exit(1)
-
-    try:
-        python_executable = json.loads(stdout.strip())
-        python_executable = python_executable[options.vm_name]
         if options.test_with_python3:
             python_executable = '/usr/bin/python3'
-        setattr(options, 'minion_python_executable', python_executable)
-        save_state(options)
-        return python_executable
-    except (ValueError, TypeError):
-        print_bulleted(
-            options,
-            'Failed to load any JSON from {0!r}'
-            ''.format(stdout.strip()),
-            'RED')
+        else:
+            cmd = ['salt', '--out=json', '-l', options.log_level]
+            if options.no_color:
+                cmd.append('--no-color')
+            cmd.extend([options.vm_name, 'grains.get', 'pythonexecutable'])
+
+            stdout, stderr, exitcode = run_command(cmd,
+                                                   options,
+                                                   return_output=True,
+                                                   stream_stdout=False,
+                                                   stream_stderr=False)
+            if exitcode != 0:
+                print_bulleted(
+                    options,
+                    'Failed to get the minion python executable. Exit code: {0}'
+                    ''.format(exitcode),
+                    'RED')
+                sys.exit(exitcode)
+
+            if not stdout.strip():
+                print_bulleted(
+                    options,
+                    'Failed to get the minion python executable (no output)',
+                    'RED')
+                sys.exit(1)
+
+            try:
+                python_executable = json.loads(stdout.strip())
+                python_executable = python_executable[options.vm_name]
+            except (ValueError, TypeError):
+                print_bulleted(
+                    options,
+                    'Failed to load any JSON from {0!r}'
+                    ''.format(stdout.strip()),
+                    'RED')
+
+    setattr(options, 'minion_python_executable', python_executable)
+    save_state(options)
+    return python_executable
 
 
 def delete_cloud_vm(options):
@@ -1055,9 +1057,9 @@ def check_win_minion_connected(options):
                options.vm_name, 'test.ping']
 
         # Attempt to connect to the new minion, it can take a while with a new
-        # install. We'll try 12 times (1 min)
+        # install. We'll try 24 times (12 min)
         retries = 0
-        while retries <= 12:
+        while retries <= 24:
 
             retries += 1
             stdout, stderr, exitcode = run_command(
@@ -1069,7 +1071,7 @@ def check_win_minion_connected(options):
                     'Failed to return a ping from the minion. Exit code: {0}'
                     ''.format(exitcode), 'RED'
                 )
-                if retries > 12:
+                if retries > 24:
                     sys.exit(exitcode)
 
             if not stdout.strip():
@@ -1077,7 +1079,7 @@ def check_win_minion_connected(options):
                     options,
                     'Failed to return a ping from the minion (no output).',
                     'RED')
-                if retries > 12:
+                if retries > 24:
                     sys.exit(1)
 
             try:
@@ -1093,12 +1095,12 @@ def check_win_minion_connected(options):
                     'Failed to load any JSON from {0!r}'.format(stdout.strip()),
                     'RED')
 
-                if retries <= 12:
+                if retries <= 24:
                     print_bulleted(
                         options,
-                        'Trying again in 5 seconds. Retry {0}'.format(retries),
+                        'Trying again in 30 seconds. Retry {0}'.format(retries),
                         'RED')
-                    time.sleep(5)
+                    time.sleep(30)
 
                 print_flush('\n')
                 continue
@@ -1118,7 +1120,7 @@ def check_win_minion_connected(options):
                         'Failed to reboot the minion. Exit code: {0}'
                         ''.format(exitcode), 'RED'
                     )
-                    if retries > 12:
+                    if retries > 24:
                         sys.exit(1)
 
                 if not stdout.strip():
@@ -1126,7 +1128,7 @@ def check_win_minion_connected(options):
                         options,
                         'Failed to reboot the minion (no output).',
                         'RED')
-                    if retries > 12:
+                    if retries > 24:
                         sys.exit(1)
 
                 try:
@@ -1167,23 +1169,23 @@ def check_win_minion_connected(options):
                 print_bulleted(options, 'ATTENTION!!!!', 'YELLOW')
                 print_bulleted(options, 'The minion did not return.', 'YELLOW')
 
-                if retries <= 12:
+                if retries <= 24:
                     print_bulleted(
                         options,
-                        'Trying again in 5 seconds. Retry {0}'
+                        'Trying again in 30 seconds. Retry {0}'
                         ''.format(retries),
                         'YELLOW')
-                    time.sleep(5)
+                    time.sleep(30)
 
                 print_flush('\n')
 
         # Ping the minion until it stops returning pings.
-        # We'll try 12 times (6 min)
+        # We'll try 24 times (12 min)
         cmd = ['salt', '--out=json', '-l', options.log_level,
                options.vm_name, 'test.ping']
 
         retries = 0
-        while retries <= 12:
+        while retries <= 24:
 
             retries += 1
             stdout, stderr, exitcode = run_command(
@@ -1195,7 +1197,7 @@ def check_win_minion_connected(options):
                     'Failed to return a ping from the minion. Exit code: {0}'
                     ''.format(exitcode), 'RED'
                 )
-                if retries > 12:
+                if retries > 24:
                     sys.exit(exitcode)
 
             if not stdout.strip():
@@ -1203,7 +1205,7 @@ def check_win_minion_connected(options):
                     options,
                     'Failed to return a ping from the minion (no output).',
                     'RED')
-                if retries > 12:
+                if retries > 24:
                     sys.exit(1)
 
             try:
@@ -1219,7 +1221,7 @@ def check_win_minion_connected(options):
                     'Failed to load any JSON from {0!r}'.format(stdout.strip()),
                     'RED')
 
-                if retries <= 12:
+                if retries <= 24:
                     print_bulleted(
                         options,
                         'Trying again in 30 seconds. Retry {0}'.format(retries),
@@ -1254,7 +1256,7 @@ def check_win_minion_connected(options):
            options.vm_name, 'grains.items']
 
     retries = 0
-    while retries <= 12:
+    while retries <= 24:
         retries += 1
         stdout, stderr, exitcode = run_command(
             cmd, options, return_output=True, stream_stdout=False,
@@ -1265,7 +1267,7 @@ def check_win_minion_connected(options):
                 'Failed to load grains from the minion. Exit code: {0}'
                 ''.format(exitcode),
                 'RED')
-            if retries > 12:
+            if retries > 24:
                 sys.exit(exitcode)
 
         if not stdout.strip():
@@ -1273,7 +1275,7 @@ def check_win_minion_connected(options):
                 options,
                 'Failed to load grains from the minion (no output).',
                 'RED')
-            if retries > 12:
+            if retries > 24:
                 sys.exit(1)
 
         try:
@@ -1290,7 +1292,7 @@ def check_win_minion_connected(options):
                 'Failed to load any JSON from {0!r}'.format(stdout.strip()),
                 'RED')
 
-            if retries <= 12:
+            if retries <= 24:
                 print_bulleted(
                     options,
                     'Trying again in 5 seconds. Retry {0}'.format(retries),
@@ -1327,7 +1329,7 @@ def check_win_minion_connected(options):
             print_bulleted(options, 'ATTENTION!!!!', 'YELLOW')
             print_bulleted(options, 'The minion did not return.', 'YELLOW')
 
-            if retries <= 12:
+            if retries <= 24:
                 print_bulleted(
                     options,
                     'Trying again in 5 seconds. Retries {0}'
@@ -1443,7 +1445,7 @@ def run_state_on_vm(options, state_name, saltenv=None, timeout=100):
     if getattr(options, 'require_sudo', False) and not options.windows:
         cmd.insert(0, 'sudo')
 
-    cmd.extend(['state.sls', state_name ])
+    cmd.extend(['state.sls', state_name])
     if options.windows:
         cmd.append(
             # This needed to get the formatting correct '"'"'C:\temp'"'"'
@@ -1476,7 +1478,7 @@ def check_cloned_reposiory_commit(options):
     cmd.extend([
         options.vm_name,
         'git.revision',
-        '/{0}'.format(options.package_source_dir)
+        '{0}'.format(options.package_source_dir)
     ])
 
     stdout, stderr, exitcode = run_command(cmd,
@@ -1589,9 +1591,9 @@ def run_winexe_command(options, remote_command):
     )
     if isinstance(remote_command, list):
         remote_command = ' '.join(remote_command)
-    cmd = 'winexe {0} \'cmd /c {1}\'' \
+    cmd = 'winexe {0} \'cmd /c "{1}"\'' \
           ''.format(credentials, remote_command)
-    logging_cmd = 'winexe {0} \'cmd /c {1}\'' \
+    logging_cmd = 'winexe {0} \'cmd /c "{1}"\'' \
                   ''.format(logging_credentials, remote_command)
     print_bulleted(options, 'Running WinEXE command: {0}'.format(logging_cmd))
     return win_cmd(cmd, logging_command=logging_cmd)
@@ -1775,7 +1777,7 @@ def build_default_test_command(options):
             '{0}\\tests\\runtests.py'.format(options.package_source_dir))
     else:
         test_command.append(
-            '/{0}/tests/runtests.py'.format(options.package_source_dir))
+            '{0}/tests/runtests.py'.format(options.package_source_dir))
     # Append basic command parameters
     test_command.extend([
         '-v',
@@ -1800,6 +1802,9 @@ def build_default_test_command(options):
             ''.format(options.package_source_dir))
     test_command.append('--xml=/tmp/xml-unittests-output')
 
+    if options.test_with_bash is True:
+        test_command = ['/bin/bash', '-li', '-c', '"{0}"'.format(' '.join(test_command))]
+
     return test_command
 
 
@@ -2088,6 +2093,12 @@ def get_args():
         action='store_true',
         help='Use python3 instead of python2 to run the test suite'
     )
+    testing_source_options.add_argument(
+        '--test-with-bash',
+        default=False,
+        action='store_true',
+        help='Use bash instead of execing python across ssh'
+    )
     testing_source_options.add_argument(
         '--test-prep-sls',
         default=[],
